import ssl
import json
import yaml
import time
from awscrt import io, mqtt, auth, http
from awsiot import mqtt_connection_builder


from datetime import datetime, timezone, timedelta

class sdtcloudMqtt():
    def __init__(self):
        appId, deviceConfig, appGroupId = self.getConfig()
        self.url = "avk03ee629rck-ats.iot.ap-northeast-2.amazonaws.com"
        self.certificate = f"/etc/sdt/cert/{deviceConfig['projectcode']}-certificate.pem"
        self.privateKey = f"/etc/sdt/cert/{deviceConfig['projectcode']}-private.pem"
        self.rootCa = f"/etc/sdt/cert/AmazonRootCA1.pem"
        if deviceConfig['devicetype'] == "aquarack":
            self.topic = f"{deviceConfig['servicecode']}/{deviceConfig['projectcode']}/{deviceConfig['assetcode']}/bwc/aquarack"
        else:
            self.topic = f"{deviceConfig['servicecode']}/{deviceConfig['projectcode']}/{deviceConfig['assetcode']}/app/{appId}/data"
        self.req_topic = f"{deviceConfig['servicecode']}/{deviceConfig['projectcode']}/{deviceConfig['assetcode']}/app/{appId}/inference-data"
        self.mqttClient = None
        self.appId = appId
        self.appGroupId = appGroupId
        self.assetCode = deviceConfig['assetcode']

    def getConfig(self):

        with open('/etc/sdt/device.config/config.json', encoding='UTF-8') as f:
                    deviceConfig = json.load(f)

        appId = ""
        appGroupId = ""

        if deviceConfig['devicetype'] == "aquarack":
            appConfig = {}
        else:
            with open('/etc/sdt/device.config/app.json', encoding='UTF-8') as f:
                appConfig = json.load(f)
            for appInfo in appConfig["AppInfoList"]:
                        if appInfo["AppName"] == appFramework["spec"]["appName"]:
                            appId = appInfo["AppId"]
                            appGroupId = appInfo["AppGroupId"]
        
        with open('./framework.yaml', encoding='UTF-8') as f:
            appFramework = yaml.load(f, Loader=yaml.FullLoader)

        return appId, deviceConfig, appGroupId

    def reconnect(self):
        attempt = 0
        while True:
            try:
                print(f"Trying to reconnect... Attempt {attempt + 1}")
                self.mqttClient.connect().result()
                print("Reconnected successfully.")
                break
            except Exception as e:
                print(f"Reconnection failed: {e}")
                attempt += 1
                # 재연결 간격을 조정할 수 있습니다. (예: 5초 대기 후 재시도)
                time.sleep(5)

    def on_connection_interrupted(self, connection, error, **kwargs):
        print(f"Connection interrupted: {error}")
        self.reconnect()

    # 연결 복원 이벤트 핸들러 추가
    def on_connection_resumed(self, connection, return_code, session_present, **kwargs):
        print(f"Connection resumed: return_code={return_code}, session_present={session_present}")

    def setClient(self, clientId):
        eventLoopGroup = io.EventLoopGroup(1)
        hostResolver = io.DefaultHostResolver(eventLoopGroup)
        clientBootstrap = io.ClientBootstrap(eventLoopGroup, hostResolver)
        self.mqttClient = mqtt_connection_builder.mtls_from_path(
            endpoint=self.url,
            cert_filepath=self.certificate,
            pri_key_filepath=self.privateKey,
            client_bootstrap=clientBootstrap,
            ca_filepath=self.rootCa,
            client_id=clientId,
            clean_session=True,
            keep_alive_secs=60)

        self.mqttClient.on_connection_interrupted = self.on_connection_interrupted
        self.mqttClient.on_connection_resumed = self.on_connection_resumed

        connectFuture = self.mqttClient.connect()
        connectFuture.result()

    def pubMessage(self, msg):
        payload = {
            "data": msg,
            "timestamp": int(time.time()*1000)
        }
        self.mqttClient.publish(topic=self.topic, payload=json.dumps(payload), qos=mqtt.QoS.AT_LEAST_ONCE)

    def aquarackPubMessage(self, msg):
            payload = {
                "data": msg,
                "timestamp": int(time.time()*1000)
            }
            self.mqttClient.publish(topic=self.topic, payload=json.dumps(payload), qos=mqtt.QoS.AT_LEAST_ONCE)

    def pubReqMessage(self, msg):
        payload = {
            "data": msg,
            "assetCode": self.assetCode,
            "appId": self.appId,
            "appGroupId": self.appGroupId,
            "timestamp": int(time.time()*1000)
        }
        self.mqttClient.publish(topic=self.req_topic, payload=json.dumps(payload), qos=mqtt.QoS.AT_LEAST_ONCE)

